@@ -5,6 +5,10 @@ import base64
 import contextlib
 import logging
 import re
+from typing import TYPE_CHECKING
+
+if TYPE_CHECKING:
+    from bleak import BleakGATTServiceCollection
 
 logger = logging.getLogger(__name__)
 
@@ -16,7 +20,7 @@ def decode_byte_string(data: bytes | bytearray) -> str:
     with contextlib.suppress(ValueError):
         b64_as_str = base64.encodebytes(data).decode("utf-8")
         return re.sub("[\r\n]", "", b64_as_str)
-    logger.warning(f'Failed to decode bytes "{data!r}". Forcing to string.')
+    logger.warning('Failed to decode bytes "%s". Forcing to string.', data)
     return str(data)
 
 
@@ -36,9 +40,23 @@ def bytes_to_big_int(data: bytearray | bytes) -> int:
 
 
 def temp_from_bytes(temp_bytes: bytearray, metric: bool = True) -> float:
-    """Get temperature from bytearray and convert to fahrenheit if needed."""
+    """Get temperature from bytearray and convert to Fahrenheit if needed."""
     temp = float(bytes_to_little_int(temp_bytes)) * 0.01
     if metric is False:
         # Convert to fahrenheit
         temp = (temp * 9 / 5) + 32
     return round(temp, 2)
+
+
+def log_services(services: BleakGATTServiceCollection) -> None:
+    """Log services for debugging."""
+    logger.debug("Logging all services that were discovered")
+    for service in services:
+        logger.debug(
+            "Service '%s' (UUID: %s) has the characteristics:\n %s",
+            service.description,
+            service.uuid,
+            "\n".join(
+                f'{characteristic.uuid}: {characteristic.description}' for characteristic in service.characteristics
+            ),
+        )
